"""
Author: Espoir Loémba

This module provides functionality for creating new Nexy projects via the command line interface.
It includes the ProjectManager class, which handles project creation, configuration, and success messaging.
"""

from os import path
from sys import platform
from typing import Optional
from typing_extensions import Annotated
from typer import Argument
from InquirerPy import inquirer
from rich.prompt import Prompt

from nexy.cli.core.constants import Console, CMD
from nexy.cli.core.models import ORM, Database, ProjectType, TestFramework
from nexy.cli.core.project_builder import ProjectBuilder
from pathlib import Path

class ProjectManager:
    def __init__(self, project_name: Optional[str] = None):
        self.project_name = project_name
        self.builder = None

    def print_success_message(self, test_framework: TestFramework):
        """Displays a success message after project creation."""
        activation_command = ".venv/Scripts/activate" if platform == "win32" else "source .venv/bin/activate"
        success_message = f"[bold green]✨ Project created successfully![/bold green]\n\nTo get started:\n[yellow]cd {self.project_name}\n{activation_command}\nnexy dev\n[/yellow]"

        if test_framework != TestFramework.NONE:
            test_commands = {
                TestFramework.PYTEST: "pytest",
                TestFramework.UNITTEST: "python -m unittest discover tests",
                TestFramework.ROBOT: "robot tests/",
            }
            success_message += f"To run tests:\n[yellow]{test_commands[test_framework]}[/yellow]\n"

        Console.print(success_message)

    def collect_project_options(self):
        """Collects project configuration options via prompts."""
        
        # Project Type
        project_type = ProjectType(inquirer.select(
            message="🤔 Starter kit: ",
            choices=[t.value for t in ProjectType],
            default=ProjectType.DEFAULT.value
        ).execute())
        self.builder.set_project_type(project_type)

        if project_type == ProjectType.DEFAULT:
            self.configure_webapp_options()

        # Database
        template_engine = Database(inquirer.select(
            message="Which database would you like to use: ",
            choices=[db.value for db in Database],
            default=Database.MYSQL.value
        ).execute())
        self.builder.set_database(template_engine)
        
        # ORM
        if template_engine != Database.NONE:
            orm = ORM(inquirer.select(
                message="Which ORM would you like to use: ",
                choices=[orm.value for orm in ORM],
                default=ORM.PRISMA.value
            ).execute())
            self.builder.set_orm(orm)

        # Test Framework
        test_framework = TestFramework(inquirer.select(
            message="Test framework to use:",
            choices=[tf.value for tf in TestFramework],
            height=20,
            default=TestFramework.PYTEST.value
        ).execute())
        self.builder.set_test_framework(test_framework)

        # Additional Features
        # self.builder.add_feature("validation")
        # if inquirer.confirm(message="Voulez-vous ajouter le support CORS?").execute():
        #     self.builder.add_feature("cors")
        # if project_type == ProjectType.MICROSERVICE and inquirer.confirm(
        #     message="Voulez-vous ajouter la documentation Swagger?"
        # ).execute():
        #     self.builder.add_feature("swagger")

    def configure_webapp_options(self):
        """Configures options specific to webapp projects."""
        template_engine = Database(inquirer.select(
            message="Which database would you like to use: ",
            choices=[db.value for db in Database],
            default=Database.MYSQL.value
        ).execute())
        self.builder.set_database(template_engine)
        if inquirer.confirm(message="Would you like to use Tailwind CSS?").execute():
            self.builder.add_feature("tailwind")

    def verify_project_name(self) -> str:
        """
        Checks if the project name is already in use.
        If so, asks the user if they want to choose a different name.
        Returns the validated project name or None if the user cancels.
        """
        while True:
            if not self.project_name:
                self.project_name = Console.input("✅ What is your project named?... ")
            else:
                Console.print(f"✅ Project name: [green]{self.project_name}[/green]\n")

            if path.isdir(self.project_name):
                Console.print(f"[red]❌ This project name already exists.[/red]")
                if not inquirer.confirm(
                    message="Do you want to choose a different name?...",
                    qmark="🤔",
                    default=True
                ).execute():
                    return None
                self.project_name = None
            else:
                return self.project_name

    def create_project(self):
        """Common function to create a new project."""
        from nexy.cli.core.utils import print_banner
        
        # print_banner()
        
        name = self.verify_project_name()
        if name is None:
            return None

        self.builder = ProjectBuilder(name)
        self.collect_project_options()

        Console.print("\n[bold green]Creating project...[/bold green]")
        self.builder.build()

        self.print_success_message(self.builder.test_framework)

@CMD.command()
def new(project_name: Annotated[Optional[str], Argument(..., help="Project name")] = None):
    """Creates a new Nexy project."""
    manager = ProjectManager(project_name)
    manager.create_project()

@CMD.command()
def n(project_name: Annotated[Optional[str], Argument(..., help="Project name")] = None):
    """Alias for the new command."""
    manager = ProjectManager(project_name)
    manager.create_project() 



import sys
import subprocess

def upgrade():
    """
    Upgrades Nexy to the latest version using pip
    """
    try:
        subprocess.check_call([sys.executable, "-m", "pip", "install", "nexy","--force"])
        print("✨ Nexy has been upgraded successfully!")
    except subprocess.CalledProcessError as e:
        print(f"❌ Error upgrading Nexy: {e}")
        sys.exit(1)






"""
Author: Espoir Loém
"""

import importlib
from os import makedirs, name, path
import subprocess
from pathlib import Path
from socket import AF_INET, SOCK_STREAM, socket
import sys
from time import sleep
from typing import List
from rich.columns import Columns
from rich.panel import Panel
from rich.progress import Progress, SpinnerColumn, TextColumn
from typer import Exit, echo

from nexy.cli.core.constants import Console
from nexy.cli.core.models import ORM, Database, ProjectType, TestFramework


def print_banner():
    Console.print(
"""
[green]
       _   __                
      / | / /__  _  ____  __
     /  |/ / _ \\| |/_/ / / /
    / /|  /  __/>  </ /_/ / 
   /_/ |_/\\___/_/|_|\\__, /  
                   /____/   
[/green]
"""
)


# Fonctions utilitaires pour la gestion des ports
def is_port_in_use(port: int, host: str = 'http://localhost') -> bool:
    with socket(AF_INET, SOCK_STREAM) as s:
        try:
            s.bind((host, port))
            return False
        except OSError:
            return True

def find_available_port(start_port: int = 3000, host: str = 'http://localhost') -> list[int]:
    available_ports = []
    current_port = start_port
    while len(available_ports) < 5 and current_port < 65535:
        if not is_port_in_use(current_port, host):
            available_ports.append(current_port)
        current_port += 1
    return available_ports

def display_port_choices(available_ports: list[int], host: str) -> None:
    panels = []
    for i, port in enumerate(available_ports, 1):
        panel_content = f"[bold]Port {port}[/bold]\n{host}:{port}"
        color = ["green", "blue", "yellow", "cyan", "magenta"][i - 1]
        panels.append(Panel(
            panel_content,
            title=f"Choix {i}",
            border_style=color,
            padding=1
        ))
    Console.print(Columns(panels))


# Fonctions de génération de fichiers
def generate_requirements(project_type: ProjectType, database: Database, orm: ORM, 
                        test_framework: TestFramework, features: List[str]) -> str:
    requirements = [
        "nexy",
        "uvicorn",
        "python-dotenv",
        "inquirerpy==0.3.4"
    ]
    
    if database != Database.NONE:
        db_requirements = {
            Database.MYSQL: ["mysql-connector-python"],
            Database.POSTGRESQL: ["psycopg2-binary"],
            Database.MONGODB: ["motor"],
            Database.SQLITE: [],
        }
        requirements.extend(db_requirements.get(database, []))
    
    if orm != ORM.NONE:
        orm_requirements = {
            ORM.PRISMA: ["prisma"],
            ORM.SQLALCHEMY: ["sqlalchemy"],
        }
        requirements.extend(orm_requirements.get(orm, []))
    
    if test_framework != TestFramework.NONE:
        test_requirements = {
            TestFramework.PYTEST: ["pytest", "pytest-cov", "pytest-asyncio"],
            TestFramework.UNITTEST: ["unittest2"],
            TestFramework.ROBOT: ["robotframework"],
        }
        requirements.extend(test_requirements.get(test_framework, []))
    
    if project_type == ProjectType.DEFAULT:
        requirements.extend(["jinja2"])
    
    if "auth" in features:
        requirements.extend(["python-jose[cryptography]", "passlib[bcrypt]"])
    
    return "\n".join(requirements)

def generate_env_file(database: Database) -> str:
    env_vars = [
        "APP_ENV=development",
        "SECRET_KEY=your-secret-key-here",
    ]
    
    db_vars = {
        Database.MYSQL: ["DATABASE_URL=mysql://user:password@localhost:3306/dbname"],
        Database.POSTGRESQL: ["DATABASE_URL=postgresql://user:password@localhost:5432/dbname"],
        Database.MONGODB: ["DATABASE_URL=mongodb://localhost:27017/dbname"],
        Database.SQLITE: ["DATABASE_URL=sqlite:///./sql_app.db"],
    }
    
    if database != Database.NONE:
        env_vars.extend(db_vars.get(database, []))
    # 
    return "\n".join(env_vars)

def create_test_config(project_name: str, test_framework: TestFramework):
    if test_framework == TestFramework.PYTEST:
        pytest_ini = "[pytest]\ntestpaths = tests\npython_files = test_*.py\npython_classes = Test\npython_functions = test_*\naddopts = -v --cov=app"
        with open(path.join(project_name, "pytest.ini"), "w") as f:
            f.write(pytest_ini)
            
        test_example = """import pytest\nfrom fastapi.testclient import TestClient\nfrom main import app\n\nclient = TestClient(app)\n\ndef test_read_main():\n    response = client.get("/")\n    assert response.status_code == 200"""
        makedirs(path.join(project_name, "tests"), exist_ok=True)
        with open(path.join(project_name, "tests", "test_main.py"), "w") as f:
            f.write(test_example)

    elif test_framework == TestFramework.ROBOT:
        robot_test = """*** Settings ***\nDocumentation Example Test Suite\nLibrary RequestsLibrary\n\n*** Test Cases ***\nTest Main Page\n Create Session app http://localhost:3000\n ${response}= GET On Session app /\n Status Should Be 200 ${response}"""
        makedirs(path.join(project_name, "tests"), exist_ok=True)
        with open(path.join(project_name, "tests", "main.robot"), "w") as f:
            f.write(robot_test)

def setup_virtualenv(project_name: str, env_name: str, requirements_file: str = None):
    """Set up a virtual environment and install dependencies.
    
    Args:
        project_name: Name of the project directory
        env_name: Name of the virtual environment
        requirements_file: Optional path to requirements.txt file
    """
    try:
        venv_path = path.join(project_name, env_name)
        
        # Check if environment already exists
        if path.exists(venv_path):
            Console.print(f"[yellow]L'environnement {env_name} existe déjà.[/yellow]")
            return

        # Create virtual environment
        Console.print(f"[blue]Création de l'environnement virtuel {env_name}...[/blue]")
        result = subprocess.run(
            [sys.executable, '-m', 'venv', venv_path],
            capture_output=True,
            text=True,
            check=True
        )

        # Get pip path based on OS
        pip_executable = path.join(venv_path, 'Scripts', 'pip.exe') if sys.platform == "win32" else path.join(venv_path, 'bin', 'pip')
        python_executable = path.join(venv_path, 'Scripts', 'python.exe') if sys.platform == "win32" else path.join(venv_path, 'bin', 'python')
        
        # Upgrade pip using python -m pip to avoid permission issues
        Console.print("[bold yellow]Mise à jour de pip...[/bold yellow]")
        subprocess.run([python_executable, '-m', 'pip', 'install', '--upgrade', 'pip'], check=True)

        # Install dependencies if requirements file exists
        if requirements_file:
            Console.print(f"[bold yellow]Installation des dépendances depuis {requirements_file}...[/bold yellow]")
            subprocess.run([pip_executable, 'install', '-r', requirements_file], check=True)
            Console.print(f"[green]✓ Dépendances installées avec succès[/green]")

        Console.print(f"[green]✓ Environnement virtuel {env_name} créé avec succès[/green]")

    except subprocess.CalledProcessError as e:
        Console.print(f"[red]Erreur lors de l'exécution de la commande : {e.stderr}[/red]")
        raise Exit(1)
    except Exception as e:
        Console.print(f"[red]Une erreur est survenue : {str(e)}[/red]")
        raise Exit(1)



def create_project_structure(
    project_name: str,
    project_type: ProjectType,
    database: Database,
    orm: ORM,
    test_framework: TestFramework,
    features: List[str]
):
    base_dirs = [
        "",
        "app",
        "public",
        "tests",
    ]
    
    if project_type == ProjectType.WEBAPP:
        base_dirs.extend(["src/components/","src/utils/"])
    
    with Progress(
        SpinnerColumn(),
        TextColumn("[progress.description]{task.description}"),
        transient=True,
    ) as progress:
        progress.add_task("[green]Création de la structure du projet...[/green]", total=None)
        
        for dir_path in base_dirs:
            full_path = path.join(project_name, dir_path)
            makedirs(full_path, exist_ok=True)
            sleep(0.1)
        
        # Check if git is installed and initialize a git repository
        if subprocess.run(["git", "--version"], capture_output=True, text=True).returncode == 0:
            subprocess.run(["git", "init", project_name], check=True)
        else:
            Console.print("[red]Git n'est pas installé. Veuillez installer Git pour initialiser un dépôt.[/red]")

        if project_type == ProjectType.DEFAULT:
            files_to_create = {
                "nexy-config.py":"""from nexy import Nexy\nrun = Nexy()\n""",
                "app/controller.py": """from nexy import component\n\n@component(\n   imports=[]\n)\ndef Layout(children):\n\treturn {"children":children}\n@component(\n   imports=[]\n)\ndef View():\n\treturn {"name": "hello world"}""",
                "app/view.html":"generate_view()",
                "app/Layout.html":" ",
                "requirements.txt": generate_requirements(project_type, database, orm, test_framework, features),
                ".env": generate_env_file(database),
                "README.md": generate_readme(project_name, project_type, database, orm, test_framework, features),
            }
        else:
            files_to_create = {
                "nexy-config.py":"""from nexy import Nexy\nrun = Nexy()\n""",
                "app/controller.py": """async def GET():\n\treturn {"name": "hello world"}\nasync def POST():\n\treturn {"name": "hello world"}""",
                "requirements.txt": generate_requirements(project_type, database, orm, test_framework, features),
                ".env": generate_env_file(database),
                ".gitignore": "# Virtual environment\nvenv\n# Python cache\n__pycache__\n# Compiled files\n*.pyc\n",
                "README.md": generate_readme(project_name, project_type, database, orm, test_framework, features),
        }

        for file_name, content in files_to_create.items():
            with open(path.join(project_name, file_name), "w") as f:
                f.write(content)
            sleep(0.1)
        
        if test_framework != TestFramework.NONE:
            create_test_config(project_name, test_framework)

def generate_readme(project_name: str, project_type: ProjectType, database: Database, 
                   orm: ORM, test_framework: TestFramework, features: List[str]) -> str:
    testing_section = ""
    if test_framework != TestFramework.NONE:
        testing_commands = {
            TestFramework.PYTEST: "pytest",
            TestFramework.UNITTEST: "python -m unittest discover tests",
            TestFramework.ROBOT: "robot tests/",
        }
        test_command = testing_commands.get(test_framework, "")
        testing_section = f"## Tests\nPour exécuter les tests :\n```bash\n{test_command}\n```\n"

    return f"# {project_name}\n\n## Description\nProjet {project_type.value} généré avec Nexy CLI\n\n## Configuration technique\n- Type: {project_type.value}\n- Base de données: {database.value}\n- ORM: {orm.value}\n- Framework de test: {test_framework.value}\n- Fonctionnalités: {', '.join(features)}\n\n## Installation\n\n1. Cloner le projet\n```bash\ngit clone <url-du-projet>\ncd {project_name}\n```\n\n2. Installer les dépendances\n```bash\nnexy install\n```\n\n4. Lancer le serveur de développement\n```bash\nnexy dev\n```\n{testing_section}\n"

# Nouvelles fonctions pour la génération de composants
def generate_controller(name: str) -> str:
    return f"""
    async def GET():
        return {{"message": "Welcome to {name} controller"}}
    
    async def POST():
        return {{"message": "Welcome to {name} controller"}}
    
    async def PUT():
        return {{"message": f"Updated {name} {{id}}", "data": data}}
    
    async def DELETE():
        return {{"message": f"Deleted {name} {{id}}"}}
"""

def generate_service(name: str) -> str:
    return f"""class {name.capitalize()}Service:
    def __init__(self):
        # Initialize your service
        pass
    
    async def get_all(self):
        # Implement get all logic
        pass
    
    async def get_by_id(self, id: int):
        # Implement get by id logic
        pass
    
    async def create(self, data: dict):
        # Implement create logic
        pass
    
    async def update(self, id: int, data: dict):
        # Implement update logic
        pass
    
    async def delete(self, id: int):
        # Implement delete logic
        pass
"""

def generate_model(name: str) -> str:
    return f"""from sqlalchemy import Column, Integer, String, DateTime
from datetime import datetime
from nexy.app import Base

class {name.capitalize()}(Base):
    __tablename__ = "{name.lower()}s"

    id = Column(Integer, primary_key=True, index=True)
    name = Column(String, index=True)
    created_at = Column(DateTime, default=datetime.utcnow)
    updated_at = Column(DateTime, default=datetime.utcnow, onupdate=datetime.utcnow)
    
    def to_dict(self):
        return {{
            "id": self.id,
            "name": self.name,
            "created_at": self.created_at.isoformat(),
            "updated_at": self.updated_at.isoformat()
        }}
"""

def generate_component(name: str) -> str:
    component_name = name[0].capitalize() + name[1:]
    return f"""
    from nexy import Component
    @Component(imports=[])
    def {component_name}(name: str):
        return {{
            "message": f"Welcome  {{name}}"
        }}
    """

def is_port_in_use(port: int, host: str = "localhost") -> bool:
    """Vérifie si un port est déjà utilisé sur l'hôte."""
    with socket(AF_INET, SOCK_STREAM) as s:
        result = s.connect_ex((host, port))
        return result == 0  # Si le résultat est 0, cela signifie que le port est utilisé

def get_next_available_port(starting_port: int = 3000, host: str = "localhost") -> int:
    """Trouve un port disponible à partir d'un port de départ."""
    port = starting_port
    while is_port_in_use(port, host):
        port += 1
    return port



"""
Author: Espoir Loém

This module defines various Enum classes to represent different configuration options
for projects, such as project type, database, ORM, test framework, and more.
These Enums are used to standardize and simplify the selection of these options
throughout the application.
"""

from enum import Enum

class ProjectType(str, Enum):
    """Enum for different types of projects."""
    DEFAULT = "Default"
    MICROSERVICE = "Microservice"

class Database(str, Enum):
    """Enum for supported database types."""
    MYSQL = "MySQL"
    POSTGRESQL = "PostgreSQL"
    MONGODB = "MongoDB"
    SQLITE = "SQLite"
    NONE = "None"

class ORM(str, Enum): 
    """Enum for supported ORM frameworks."""
    PRISMA = "Prisma"
    SQLALCHEMY = "SQLAlchemy"
    NONE = "None"

class TestFramework(str, Enum):
    """Enum for supported test frameworks."""
    PYTEST = "Pytest"
    UNITTEST = "Unittest"
    NONE = "None"

class RouterType(str, Enum):
    """Enum for router configuration types."""
    APP_ROUTER = "App Router"
    MANUAL = "Manual Routing"

class AuthType(str, Enum):
    """Enum for authentication methods."""
    JWT = "JWT Authentication"
    SESSION = "Session Based"
    OAUTH = "OAuth 2.0"
    NONE = "None"

class CacheType(str, Enum):
    """Enum for caching strategies."""
    REDIS = "Redis"
    MEMCACHED = "Memcached"
    INMEMORY = "In-Memory"
    NONE = "None"

class Languages(str, Enum):
    """Enum for supported languages."""
    ENGLISH = "English"
    FRENCH = "Français"
    SPANISH = "Español"
    ARABIC = "العربية"
    HINDI = "हिन्दी"
    CHINESE = "中文"
    PORTUGUESE = "Português"
    GERMAN = "Deutsch"

class CssFramework(str, Enum):
    """Enum for CSS frameworks."""
    TAILWIND = "Tailwindcss"
    NONE = "None"

class TemplateEngine(str, Enum):
    """Enum for template engines."""
    JINJA2 = "Jinja2"
    MASONITE = "Masonite"
    NONE = "None"

class JsFramework(str, Enum):
    """Enum for JavaScript frameworks."""
    REACT = "React"
    NEXTJS = "Nextjs"
    NUXTJS = "Nuxtjs"
    NONE = "None"


"""
Author: Espoir Loémba

This module provides functionality for adding a package to a Nexy project via the command line interface.
"""

from subprocess import run, CalledProcessError
from os import path
from typer import Argument, Exit
from rich.progress import Progress, SpinnerColumn, TextColumn

from nexy.cli.core.constants import Console, CMD
from nexy.cli.core.utils import print_banner

@CMD.command()
def add(
    package: str = Argument(..., help="Package to add to the project")
):
    """Adds a dependency to the project."""
    # print_banner()
    
    if not path.exists("requirements.txt"):
        Console.print("[yellow]⚠️  requirements.txt file not found. Creating...[/yellow]\n")
    
    try:
        with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=True,
        ) as progress:
            progress.add_task(f"[green]Installing {package}...", total=None)
            print("\n")
            run(["pip", "install", package], check=True)
            
            # Update requirements.txt
            run(["pip", "freeze"], stdout=open("requirements.txt", "w"), check=True)
            
        Console.print(f"[green]✨ Package {package} installed and added to requirements.txt[/green]\n")
    except CalledProcessError as e:
        Console.print(f"[red]❌ Error during installation: {str(e)}[/red]")
        raise Exit(1)




"""
Author: Espoir Loémba

This module provides functionality for initializing a new Nexy project via the command line interface.
"""

import os
import subprocess
from pathlib import Path
from typing_extensions import Annotated
from typer import Argument, Option
from typing import Optional
from nexy.cli.core.constants import Console, CMD

@CMD.command()
def init(
    project_name: Annotated[Optional[str], Argument(..., help="Project name")] = None,
    template: Annotated[Optional[str], Option(None, "--template", "-t", help="Project template")] = None
):
    """
    Initializes a new Nexy project.
    """
    if not project_name:
        project_name = Console.input("✅ What is your project named?... ")

    project_path = Path(project_name)
    if project_path.exists():
        Console.print(f"[red]❌ The project directory '{project_name}' already exists.[/red]")
        return

    try:
        # Create project directory
        project_path.mkdir(parents=True, exist_ok=False)
        Console.print(f"[green]✨ Created project directory: {project_name}[/green]")

        # Optionally, clone a template repository
        if template:
            subprocess.check_call(["git", "clone", template, str(project_path)])
            Console.print(f"[green]✨ Cloned template from {template}[/green]")
        else:
            # Create a basic project structure
            (project_path / "main.py").write_text("# Your Nexy project starts here\n")
            Console.print(f"[green]✨ Created basic project structure[/green]")

        Console.print(f"[bold green]✨ Project '{project_name}' initialized successfully![/bold green]")
    except Exception as e:
        Console.print(f"[red]❌ Error initializing project: {e}[/red]")



"""
Author: Espoir Loém

This module provides a ProjectBuilder class to facilitate the creation of project structures
with specified configurations such as project type, database, ORM, test framework, and features.
"""

from typing import List, Optional
from nexy.cli.core.models import CssFramework, ProjectType, Database, ORM, TestFramework
from nexy.cli.core.utils import create_project_structure, setup_virtualenv
from InquirerPy import inquirer

class ProjectBuilder:
    def __init__(self, project_name: str):
        self.project_name = project_name
        self.project_type: Optional[ProjectType] = ProjectType.DEFAULT
        self.database: Optional[Database] = Database.NONE
        self.orm: Optional[ORM] = ORM.NONE
        self.test_framework: Optional[TestFramework] = TestFramework.NONE
        self.css_framework: Optional[CssFramework] = None
        self.features: List[str] = []

    def set_project_type(self, project_type: ProjectType) -> 'ProjectBuilder':
        self.project_type = project_type
        return self

    def set_database(self, database: Database) -> 'ProjectBuilder':
        self.database = database
        return self

    def set_orm(self, orm: ORM) -> 'ProjectBuilder':
        self.orm = orm
        return self

    def set_test_framework(self, test_framework: TestFramework) -> 'ProjectBuilder':
        self.test_framework = test_framework
        return self

    def set_css_framework(self, css_framework: CssFramework) -> 'ProjectBuilder':
        self.css_framework = css_framework
        return self

    def add_feature(self, feature: str) -> 'ProjectBuilder':
        self.features.append(feature)
        return self

    def build(self) -> None:
        """Creates the project structure with the specified configurations."""
        self.add_feature("validation")
        # if inquirer.confirm(message="Voulez-vous ajouter le support CORS?").execute():
        #     self.add_feature("cors")
        # if self.project_type == ProjectType.MICROSERVICE and inquirer.confirm(
        #     message="Voulez-vous ajouter la documentation Swagger?"
        # ).execute():
        #     self.add_feature("swagger")

        create_project_structure(
            project_name=self.project_name,
            project_type=self.project_type,
            database=self.database,
            orm=self.orm,
            test_framework=self.test_framework,
            features=self.features
        )
        setup_virtualenv(self.project_name, '.venv', f'{self.project_name}/requirements.txt')

# Example usage:
# builder = ProjectBuilder("MyProject")
# builder.set_project_type(ProjectType.WEB).set_database(Database.POSTGRESQL).add_feature("authentication").build()